use libc::fork;
use log::{debug, trace};
use manifest::Manifest;
use pkgfile::PKGFile;
use crate::tmp::TempDir;

#[derive(Debug)]
pub enum InstallType {
    Build,
    Bin
}

#[derive(Debug)]
pub enum InstallError {
    BuildError(String),
    BinError(String),
    InstallError
}

impl ToString for InstallError {
    fn to_string(&self) -> String {
        match self {
            InstallError::BuildError(e) => format!("Build error: {}", e),
            InstallError::BinError(e) => format!("Bin error: {}", e),
            InstallError::InstallError => "Install error".to_string(),
        }
    }
}

#[derive(Debug)]
pub struct PackageInstaller {
    manifest: Manifest,
    pkgfile: PKGFile,
    install_type: InstallType
}

impl PackageInstaller {
    pub fn new(m: Manifest, p: PKGFile, i: InstallType) -> PackageInstaller {
        PackageInstaller {
            manifest: m,
            pkgfile: p,
            install_type: i
        }
    }

    fn extract_to<S: Into<String>>(&self, path: S) -> Result<(), String> {
        tar::Archive::new(self.pkgfile.data.as_slice())
            .unpack(path.into())
            .map_err(|e| e.to_string())
    }

    fn bin(&self) -> Result<(), String> {
        let mut tmpdir = TempDir::default();
        tmpdir.push(&self.manifest.package.name);
        trace!("extracting package into: {}", tmpdir.to_string());
        self.extract_to(tmpdir.to_string())?;
        debug!("extracted package in: {}", tmpdir.to_string());
        Ok(())
    }

    fn build(&self) -> Result<(), String> {
        Ok(())
    }

    pub fn install(&self) -> Result<(), InstallError> {
        match self.install_type {
            InstallType::Bin => self.bin().map_err(|e| InstallError::BinError(e)),
            InstallType::Build => self.build().map_err(|e| InstallError::BuildError(e))
        }
    }
}